namespace SettingRenderer {
    void Render(Meta::PluginSetting@ setting) {
        RenderValue(setting);
        RenderLabel(setting);
    }

    void RenderValue(Meta::PluginSetting@ setting) {
        switch(setting.Type) {
            case Meta::PluginSettingType::Bool:
                RenderBool(setting);
                break;
            case Meta::PluginSettingType::Enum:
                RenderEnum(setting);
                break;
            case Meta::PluginSettingType::Float:
                RenderFloat(setting);
                break;
            case Meta::PluginSettingType::Int8:
                RenderInt8(setting);
                break;
            case Meta::PluginSettingType::Int16:
                RenderInt16(setting);
                break;
            case Meta::PluginSettingType::Int32:
                RenderInt32(setting);
                break;
            case Meta::PluginSettingType::Uint8:
                RenderUint8(setting);
                break;
            case Meta::PluginSettingType::Uint16:
                RenderUint16(setting);
                break;
            case Meta::PluginSettingType::Uint32:
                RenderUint32(setting);
                break;
            case Meta::PluginSettingType::String:
                RenderString(setting);
                break;
            case Meta::PluginSettingType::Vec2:
                RenderVec2(setting);
                break;
            case Meta::PluginSettingType::Vec3:
                RenderVec3(setting);
                break;
            case Meta::PluginSettingType::Vec4:
                RenderVec4(setting);
                break;
            default:
                UI::TextDisabled("Unsupported setting type");
                break;
        }
    }

    void RenderLabel(Meta::PluginSetting@ setting) {
        UI::SameLine();
        UI::Text(setting.Name);
        if (setting.Description.Length > 0) {
            UI::SameLine();
            UI::TextDisabled(Icons::QuestionCircle);
            if (UI::IsItemHovered()) {
                UI::BeginTooltip();
                UI::Text(setting.Description);
                UI::EndTooltip();
            }
        }
    }

    void RenderBool(Meta::PluginSetting@ setting) {
        setting.WriteBool(UI::Checkbox("##" + setting.Name, setting.ReadBool()));
    }

    void RenderEnum(Meta::PluginSetting@ setting) {
        setting.WriteEnum(int32(UI::InputInt("##" + setting.Name, int(setting.ReadEnum()))));
    }

    void RenderFloat(Meta::PluginSetting@ setting) {
        setting.WriteFloat(UI::InputFloat("##" + setting.Name, setting.ReadFloat()));
    }

    void RenderInt8(Meta::PluginSetting@ setting) {
        setting.WriteInt8(int8(UI::InputInt("##" + setting.Name, int(setting.ReadInt8()))));
    }

    void RenderInt16(Meta::PluginSetting@ setting) {
        setting.WriteInt16(int16(UI::InputInt("##" + setting.Name, int(setting.ReadInt16()))));
    }

    void RenderInt32(Meta::PluginSetting@ setting) {
        setting.WriteInt32(int32(UI::InputInt("##" + setting.Name, int(setting.ReadInt32()))));
    }

    void RenderUint8(Meta::PluginSetting@ setting) {
        setting.WriteUint8(uint8(UI::InputInt("##" + setting.Name, int(setting.ReadUint8()))));
    }

    void RenderUint16(Meta::PluginSetting@ setting) {
        setting.WriteUint16(uint16(UI::InputInt("##" + setting.Name, int(setting.ReadUint16()))));
    }

    void RenderUint32(Meta::PluginSetting@ setting) {
        setting.WriteUint32(uint32(UI::InputInt("##" + setting.Name, int(setting.ReadUint32()))));
    }

    void RenderString(Meta::PluginSetting@ setting) {
        setting.WriteString(UI::InputText("##" + setting.Name, setting.ReadString()));
    }

    void RenderVec2(Meta::PluginSetting@ setting) {
        setting.WriteVec2(UI::InputFloat2("##" + setting.Name, setting.ReadVec2()));
    }

    void RenderVec3(Meta::PluginSetting@ setting) {
        setting.WriteVec3(UI::InputFloat3("##" + setting.Name, setting.ReadVec3()));
    }

    void RenderVec4(Meta::PluginSetting@ setting) {
        setting.WriteVec4(UI::InputFloat4("##" + setting.Name, setting.ReadVec4()));
    }
} 