class SettingsWindow {
    private bool visible = false;
    private Meta::Plugin@ plugin = null;
    private Meta::PluginSetting@ settingToFocus = null;
    private Meta::PluginSetting@ settingToHighlight = null;

    bool get_Visible() { return visible; }
    void set_Visible(bool value) { visible = value; if (!visible) { @settingToFocus = null; @settingToHighlight = null; } }
    Meta::Plugin@ get_Plugin() { return plugin; }
    void set_Plugin(Meta::Plugin@ value) { @plugin = value; }
    Meta::PluginSetting@ get_SettingToFocus() { return settingToFocus; }
    void set_SettingToFocus(Meta::PluginSetting@ value) { @settingToFocus = value; @settingToHighlight = value; }

    void Render() {
        if (!visible) return;

        if (plugin is null) {
            Visible = false;
            return;
        }

        UI::SetNextWindowSize(600, 400, UI::Cond::FirstUseEver);
        if (UI::Begin("\\$fff" + Icons::Cog + "\\$z " + plugin.Name + "###PluginSettings", Visible)) {
            RenderTabs();
        }
        UI::End();

        // Reset setting to focus after first render
        if (settingToFocus !is null) {
            @settingToFocus = null;
        }
    }

    private void RenderTabs() {
        auto settings = GetVisibleSettings(plugin);
        auto categories = GroupSettingsByCategory(settings);
        
        UI::BeginTabBar("SettingsTabs");
        auto categoriesKeys = categories.GetKeys();
        for (uint i = 0; i < categoriesKeys.Length; i++) {
            string category = categoriesKeys[i];
            auto categorySettings = cast<array<Meta::PluginSetting@>>(categories[category]);
            
            bool shouldFocus = false;
            if (settingToFocus !is null) {
                for (uint j = 0; j < categorySettings.Length; j++) {
                    if (categorySettings[j] is settingToFocus) {
                        shouldFocus = true;
                        break;
                    }
                }
            }
            
            RenderCategoryTab(category, categorySettings, shouldFocus);
        }
        UI::EndTabBar();
    }

    private dictionary GroupSettingsByCategory(array<Meta::PluginSetting@> settings) {
        dictionary categories;
        for (uint i = 0; i < settings.Length; i++) {
            string category = settings[i].Category;
            if (category.Length == 0) {
                category = "Uncategorized";
            }
            
            if (!categories.Exists(category)) {
                array<Meta::PluginSetting@> categorySettings;
                categories[category] = categorySettings;
            }
            array<Meta::PluginSetting@>@ categorySettings = cast<array<Meta::PluginSetting@>>(categories[category]);
            categorySettings.InsertLast(settings[i]);
        }
        return categories;
    }

    private void RenderCategoryTab(const string &in category, array<Meta::PluginSetting@>@ settings, bool open) {
        if (UI::BeginTabItem(category, open ? UI::TabItemFlags::SetSelected : UI::TabItemFlags::None)) {
            if (UI::BeginChild("Settings_" + category)) {
                if (UI::Button("Reset to default")) {
                    for (uint j = 0; j < settings.Length; j++) {
                        settings[j].Reset();
                    }
                }
                for (uint j = 0; j < settings.Length; j++) {
                    bool shouldFocus = settings[j] is settingToFocus;
                    if (shouldFocus) {
                        UI::SetScrollHereY();
                    }

                    bool shouldHighlight = settings[j] is settingToHighlight;
                    if (shouldHighlight) {
                        vec2 pos = UI::GetCursorPos() + UI::GetWindowPos() - vec2(0, UI::GetScrollY()) - vec2(4, 4);
                        vec2 size = vec2(UI::GetWindowContentRegionWidth(), 24) + vec2(4, 8);
                        vec4 rect = vec4(pos.x, pos.y, size.x, size.y);

                        vec2 clipRectSize = UI::GetWindowSize();
                        vec2 clipRectPos = UI::GetWindowPos();
                        vec4 clipRect = vec4(clipRectPos.x - 4, clipRectPos.y, clipRectSize.x + 8, clipRectSize.y);

                        UI::GetWindowDrawList().PushClipRect(clipRect);
                        UI::GetWindowDrawList().AddRectFilled(rect, vec4(0.2f, 0.4f, 0.8f, 0.2f), 4.0f);
                        UI::GetWindowDrawList().AddRect(rect, vec4(0.2f, 0.4f, 0.8f, 0.8f), 4.0f, 1.0f);
                        UI::GetWindowDrawList().PopClipRect();
                    }
                    SettingRenderer::Render(settings[j]);
                }
            }
            UI::EndChild();
            UI::EndTabItem();
        }
    }

    void HandleNavigation() {
        if (!visible) return;

        if (UI::IsKeyPressed(UI::Key::Escape)) {
            Visible = false;
            @plugin = null;
            return;
        }
    }
} 